import csv
from datetime import datetime, timedelta, date
import math
import astropy.time
import dateutil.parser
import json


def juliandate(correctDate):
    """
    This function converts classical dates into a Julian date.
    :param correctDate: date (datetime)
    :return: julian date (integer)
    """
    time = astropy.time.Time(correctDate)
    jd = time.jd
    return (jd)


def juliandateToDatetime(jd):
    """
    This function converts a Julian date into a classical date.
    :param jd: julian date
    :return: date (datetime)
    """
    if jd == "":
        return ("")
    time = astropy.time.Time(jd, format="jd")
    datetime = time.datetime
    return (datetime)


def dictCreator(datum, typus, calender, beforeafter, certainty, normalizedStartDate, normalizedEndDate,
                normalizedDuration):
    """
    This function transfers various information about a date into a dictionary.
    :param datum: date (datetime)
    :param typus: type of date (string)
    :param calender: julian date (integer)
    :param beforeafter: binary variable, whether a date is before or after a point in time according to Gedbas4all (string)
    :param certainty: information on the certainty of the date according to Gedbas4all (string)
    :param normalizedStartDate: julian date (integer)
    :param normalizedEndDate: julian date (integer)
    :param normalizedDuration: julian date (integer)
    :return: dictionary with this values (dictionary)
    """
    dictDate = {
        "datum": datum,  # datetime object
        "type": typus,  # point, range, relative, point in range, open ended range
        "calender": calender,  # URI
        "beforeafter": beforeafter,  # before or after
        "certainty": certainty,  # about, calculated, exact, estimated
        "normalizedStartDate": normalizedStartDate,  # int
        "normalizedEndDate": normalizedEndDate,  # int
        "normalizedDuration": normalizedDuration,  # int
    }
    return (dictDate)


def yearchecker(firstDate, secondDate):
    """
    This function isolates the year from dates as a string.
    The function is used, for example, when the year of death is to be recognized from the date of death or burial.
    :param firstDate: date with priority (string)
    :param secondDate: date (string)
    :return: year (string)
    """
    if firstDate != "":
        year = firstDate[(len(firstDate) - 4):len(firstDate)]
    elif secondDate != "":
        year = secondDate[(len(secondDate) - 4):len(secondDate)]
    else:
        year = "xxxx"
    return (year)


def datetimechecker(firstDate, secondDate):
    """
    This function writes various information of a datetime object into a list.
    If the date is not a datetime object, then it is a string that could not be converted to a datetime object.
    :param firstDate: date with priority (datetime or string)
    :param secondDate: date (datetime or string)
    :return: list of values (list)
    """
    # check first date
    if isinstance(firstDate, datetime):
        y = firstDate.year
        m = firstDate.month
        d = firstDate.day
        start = firstDate
        normalizedStartDate = juliandate(firstDate)
    # check second date
    elif isinstance(secondDate, datetime):
        y = secondDate.year
        m = secondDate.month
        d = secondDate.day
        start = secondDate
        normalizedStartDate = juliandate(secondDate)
    else:
        return  # if no datetime object, then simply return None
    values = [y, m, d, start, normalizedStartDate]
    return (values)


def adder(y, m, d, days):
    """
    This function adds days to a given date.
    :param y: year (integer)
    :param m: month (integer)
    :param d: day (integer)
    :param days: number of days to add (integer)
    :return: recalculated date (datetime)
    """
    correctDate = datetime(y, m, d) + timedelta(days=days)  # add up days
    return (correctDate)


def subtractor(y, m, d, days):
    """
    This function subtracts days from a given date.
    :param y: year (integer)
    :param m: month (integer)
    :param d: day (integer)
    :param days: number of days to subtract (integer)
    :return: recalculated date (datetime)
    """
    correctDate = datetime(y, m, d) - timedelta(days=days)  # subtract days
    return (correctDate)


def stringToDatetime(value):
    """
    This function obtains a date (datetime) from a date (string) if the format fits.
    :param value: date (string)
    :return: date(datetime, if the original format fits)
    """
    try:
        value = datetime.strptime(value, "%d.%m.%Y")
        return (value)
    except:
        return (value)


def birthbaptism(kind, death, funeral, age, days, datum, typus, calender, beforeafter, certainty, normalizedStartDate,
                 normalizedEndDate, normalizedDuration):
    """
    This function calculates birth/baptism information based on age information and the death/burial information.
    :param kind: information which date is relevant (birth, baptism)
    :param death: death date information (string)
    :param funeral: funeral date information (string)
    :param age: age in years (integer)
    :param days: age in days (integer)
    :param datum: date (datetime)
    :param typus: type of date (string)
    :param calender: julian date (integer)
    :param beforeafter: binary variable, whether a date is before or after a point in time (string)
    :param certainty: date accuracy (string)
    :param normalizedStartDate: julian date (integer)
    :param normalizedEndDate: julian date (integer)
    :param normalizedDuration: julian date (integer)
    :return: corrected date information (dictionary)
    """
    year = yearchecker(death, funeral)
    try:
        year = int(year)  # check if it is a number

        # first death then funeral is checked
        datumcorrect = 0  # auxiliary variable to determine whether the date could be converted to a datetime object
        # "death" is not a datetime here yet
        death2 = stringToDatetime(death)  # check if the date has the correct format
        funeral2 = stringToDatetime(funeral)  # check if the date has the correct format
        if isinstance(death2, datetime):
            newDate = death2
            datumcorrect = 1
        elif isinstance(funeral2, datetime):
            newDate = funeral2
            datumcorrect = 1

        if datumcorrect == 0 and kind == "J.":
            birthYear = year - age
            start1 = "01.01." + str(birthYear)
            end1 = "31.12." + str(birthYear)
            startDate = datetime.strptime(start1, "%d.%m.%Y")
            normalizedStartDate = juliandate(startDate)
            endDate = datetime.strptime(end1, "%d.%m.%Y")
            normalizedEndDate = juliandate(endDate)
            typus = "relative"
            certainty = "calculated"
            beforeafter = "before"
            datum = startDate
            # create and return the dictionary
            dictDate = dictCreator(datum, typus, calender, beforeafter, certainty, normalizedStartDate,
                                   normalizedEndDate, normalizedDuration)
            return (dictDate)

        # here the date must be split into the components
        if datumcorrect == 1:
            y = newDate.year
            m = newDate.month
            d = newDate.day
            if kind == "J.":  # by ymd can now be calculated more precisely
                dateX = datetime(y, m, d)
                birthY = dateX.replace(year=dateX.year - age)
                normalizedStartDate = juliandate(birthY)
                normalizedEndDate = juliandate(birthY.replace(year=birthY.year - 1) + timedelta(
                    days=1))  # minus one year, plus one day, since time span
                typus = "relative"
                certainty = "calculated"
                beforeafter = "before"
                datum = birthY
                # create and return the dict
                dictDate = dictCreator(datum, typus, calender, beforeafter, certainty, normalizedStartDate,
                                       normalizedEndDate, normalizedDuration)
                return (dictDate)
            correctDate = subtractor(y, m, d, days)
            datum = correctDate
            # more variables
            typus = "relative"
            certainty = "calculated"
            beforeafter = "before"
            normalizedDuration = days
            dictDate = dictCreator(datum, typus, calender, beforeafter, certainty, normalizedStartDate,
                                   normalizedEndDate, normalizedDuration)
            return (dictDate)

        # only a number of years, but not a whole date
        birthYear = year  # same year
        start1 = "01.01." + str(birthYear)
        end1 = "31.12." + str(birthYear)
        startDate = datetime.strptime(start1, "%d.%m.%Y")
        normalizedStartDate = juliandate(startDate)
        endDate = datetime.strptime(end1, "%d.%m.%Y")
        normalizedEndDate = juliandate(endDate)
        typus = "relative"
        certainty = "calculated"
        beforeafter = "before"
        datum = startDate
    except ValueError:  # no year could be found
        pass
    dictDate = dictCreator(datum, typus, calender, beforeafter, certainty, normalizedStartDate, normalizedEndDate,
                           normalizedDuration)
    return (dictDate)


def marriagedeathfuneral(kind, birth, baptism, age, days, datum, typus, calender, beforeafter, certainty,
                         normalizedStartDate, normalizedEndDate, normalizedDuration):
    """
    This function calculates death/burial information based on age information and the birth/baptism information.
    :param kind: information which date is relevant (death, funeral)
    :param birth: birth date information (string)
    :param baptism: baptism date information (string)
    :param age: age in years (integer)
    :param days: age in days (integer)
    :param datum: date (datetime)
    :param typus: type of date (string)
    :param calender: julian date (integer)
    :param beforeafter: binary variable, whether a date is before or after a point in time (string)
    :param certainty: Genauigkeit des Datums (string)
    :param normalizedStartDate: julian date (integer)
    :param normalizedEndDate: julian date (integer)
    :param normalizedDuration: julian date (integer)
    :return: corrected date information (dictionary)
    """
    # this happens when there is only information that cannot be counted on
    if datetimechecker(birth, baptism) == None:
        dictDate = dictCreator(datum, typus, calender, beforeafter, certainty, normalizedStartDate, normalizedEndDate,
                               normalizedDuration)
        return (dictDate)
    datetimecheckerResult = datetimechecker(birth, baptism)
    y = datetimecheckerResult[0]
    m = datetimecheckerResult[1]
    d = datetimecheckerResult[2]
    if kind == "J.":
        y = y + age
        correctDate = datetime(y, m, d)
    else:
        correctDate = adder(y, m, d, days)

    datum = correctDate
    certainty = "calculated"
    typus = "relative"
    beforeafter = "after"
    start = datetimechecker(birth, baptism)[3]
    normalizedStartDate = datetimechecker(birth, baptism)[4]

    # calculate the duration
    if kind == "J.":
        age = str(age)
        while len(age) <= 3:
            age = "0" + age
        duration = datetime.strptime(age, "%Y")
        normalizedDuration = (duration - datetime(1, 1, 1)).days + 365  # for the year 0
    elif kind == "T.":
        days = str(days)
        duration = datetime.strptime(days, "%d")
        normalizedDuration = (duration - datetime(1, 1, 1)).days + 365  # for the year 0
    elif kind == "ja":
        normalizedDuration = 0  # no duration

    dictDate = dictCreator(datum, typus, calender, beforeafter, certainty, normalizedStartDate, normalizedEndDate,
                           normalizedDuration)
    return (dictDate)


def problemsolver(typ, kind, birth, baptism, death, funeral, age, days, datum, typus, calender, beforeafter, certainty,
                  normalizedStartDate, normalizedEndDate, normalizedDuration):
    """
    This function prepares the correction of birth/baptism information and death/burial information.
    :param typ: designation of the date to be corrected (string)
    :param kind: information which date is relevant (birth, baptism, death, funeral)
    :param birth: birth date information (string)
    :param baptism: baptism date information (string)
    :param death: death date information (string)
    :param funeral: funeral date information (string)
    :param age: age in years (integer)
    :param days: age in days (integer)
    :param datum: date (datetime)
    :param typus: type of date (string)
    :param calender: julian date (integer)
    :param beforeafter: binary variable, whether a date is before or after a point in time (string)
    :param certainty: date accuracy (string)
    :param normalizedStartDate: julian date (integer)
    :param normalizedEndDate: julian date (integer)
    :param normalizedDuration: julian date (integer)
    :return: corrected date information (dictionary)
    """
    if typ == "birth" or typ == "baptism":
        dictDate = birthbaptism(kind, death, funeral, age, days, datum, typus, calender, beforeafter, certainty,
                                normalizedStartDate, normalizedEndDate, normalizedDuration)
        return (dictDate)
    elif typ == "marriage" or typ == "death" or typ == "funeral":
        dictDate = marriagedeathfuneral(kind, birth, baptism, age, days, datum, typus, calender, beforeafter, certainty,
                                        normalizedStartDate, normalizedEndDate, normalizedDuration)
        return (dictDate)


def corrector(typ, datecheck, birth, baptism, marriage, death, funeral, datum, typus, calender, beforeafter, certainty,
              normalizedStartDate, normalizedEndDate, normalizedDuration):
    """
    This function corrects dates.
    :param typ: designation of the date to be corrected (string)
    :param datecheck: zu korrigierendes Datum (string)
    :param birth: birth date information (string)
    :param baptism: baptism date information (string)
    :param marriage: marriage date information (string)
    :param death: death date information (string)
    :param funeral: funeral date information (string)
    :param datum: date (datetime)
    :param typus: type of date (string)
    :param calender: julian date (integer)
    :param beforeafter: binary variable, whether a date is before or after a point in time (string)
    :param certainty: date accuracy (string)
    :param normalizedStartDate: julian date (integer)
    :param normalizedEndDate: julian date (integer)
    :param normalizedDuration: julian date (integer)
    :return: corrected date information (dictionary)
    """
    # year without further information (e.g. 1800)
    try:
        datecheck2 = int(datecheck)
        start1 = "01.01." + str(datecheck2)
        end1 = "31.12." + str(datecheck2)
        startDate = datetime.strptime(start1, "%d.%m.%Y")
        normalizedStartDate = juliandate(startDate)
        endDate = datetime.strptime(end1, "%d.%m.%Y")
        normalizedEndDate = juliandate(endDate)
        typus = "point in range"
        certainty = "exact"
        datum = startDate
        dictDate = dictCreator(datum, typus, calender, beforeafter, certainty, normalizedStartDate, normalizedEndDate,
                               normalizedDuration)
        return (dictDate)
    except ValueError:
        pass

    # date with "J." (year)
    if "J." in datecheck:
        kind = "J."
        # check whether there is really a space before the J., otherwise error message
        if datecheck[datecheck.find("J.") - 1:datecheck.find("J.")] != " ":
            print("Error: A space is missing in the following entry,", datecheck)
        try:
            age = int(datecheck[:datecheck.find("J.") - 1])  # minus 1, because there is a space

        except ValueError:  # example: 60 1/2 J. (years)
            try:
                age = int(datecheck[:datecheck.find(" ")])  # everything before the space
            except ValueError:
                print("Error: The following date cannot be processed,", datecheck)
                dictDate = dictCreator(datum, typus, calender, beforeafter, certainty, normalizedStartDate,
                                       normalizedEndDate,
                                       normalizedDuration)
                return (dictDate)
        days = 0
        dictDate = problemsolver(typ, kind, birth, baptism, death, funeral, age, days, datum, typus, calender,
                                 beforeafter, certainty,
                                 normalizedStartDate, normalizedEndDate, normalizedDuration)
        return (dictDate)

    # date with "T." (day)
    if "T." in datecheck:
        kind = "T."
        age = 0
        try:
            days = int(datecheck[:datecheck.find("T.") - 1])  # minus 1, because there is a space
        except ValueError:
            try:
                days = int(datecheck[:datecheck.find(" ")])  # everything before the space
            except ValueError:
                print("Error: The following date cannot be processed,", datecheck)
                dictDate = dictCreator(datum, typus, calender, beforeafter, certainty, normalizedStartDate,
                                       normalizedEndDate,
                                       normalizedDuration)
                return (dictDate)
        dictDate = problemsolver(typ, kind, birth, baptism, death, funeral, age, days, datum, typus, calender,
                                 beforeafter, certainty,
                                 normalizedStartDate, normalizedEndDate, normalizedDuration)
        return (dictDate)

    # date is "ja" (yes)
    # that means that the person died (early)
    if "ja" in datecheck:
        kind = "ja"
        age = 0
        days = 0
        dictDate = problemsolver(typ, kind, birth, baptism, death, funeral, age, days, datum, typus, calender,
                                 beforeafter, certainty,
                                 normalizedStartDate, normalizedEndDate, normalizedDuration)
        return (dictDate)

    dictDate = dictCreator(datecheck, typus, calender, beforeafter, certainty, normalizedStartDate, normalizedEndDate,
                           normalizedDuration)
    return (dictDate)


def resolverdate(typ, datecheck, birth, baptism, marriage, death, funeral):
    """
    This function initiates the preparation of date information.
    :param typ: designation of the date to be corrected (string)
    :param datecheck: information about the date to be corrected (string)
    :param birth: birth date information (string)
    :param baptism: baptism date information (string)
    :param marriage: marriage date information (string)
    :param death: death date information (string)
    :param funeral: funeral date information (string)
    :return: corrected date information (dictionary)
    """
    # initialization of variables
    datum = ""
    typus = ""
    calender = ""
    beforeafter = ""
    certainty = ""
    normalizedStartDate = ""
    normalizedEndDate = ""
    normalizedDuration = ""
    dictDate = ""
    correctorExecuted = 0
    correct = 0

    # correction loop
    # loop is canceled when the date is successfully corrected (correct == 1)
    i = 0  # counter
    while i < 3 and correct == 0:
        try:  # try to format the date into a datetime object
            correctDate = datetime.strptime(datecheck, "%d.%m.%Y")
            datum = correctDate
            # when the correction process has been executed
            if correctorExecuted == 0:  # reicht ggf. i = 0 aus?
                typus = "point"
                certainty = "exact"
            correct = 1
        except ValueError:  # if date cannot be written in datetime a correction attempt is made
            correctDateDict = corrector(typ, datecheck, birth, baptism, marriage, death, funeral, datum, typus,
                                        calender, beforeafter, certainty, normalizedStartDate, normalizedEndDate,
                                        normalizedDuration)
            try:
                datecheck = str(correctDateDict["datum"].day) + "." + str(correctDateDict["datum"].month) + "." + str(
                    correctDateDict["datum"].year)  # for next pass of the loop
            except AttributeError:
                # then the field Date is empty, because none could be found, because error is there
                pass
            correctorExecuted = 1
            datum = correctDateDict["datum"]
            typus = correctDateDict["type"]
            calender = correctDateDict["calender"]
            beforeafter = correctDateDict["beforeafter"]
            certainty = correctDateDict["certainty"]
            normalizedStartDate = correctDateDict["normalizedStartDate"]
            normalizedEndDate = correctDateDict["normalizedEndDate"]
            normalizedDuration = correctDateDict["normalizedDuration"]
        i = i + 1

    # limit always one less than the above limit (so for 3, 2)
    if i < 2:
        calender = juliandate(correctDate)
    else:
        datum = datecheck
    dictDate = dictCreator(datum, typus, calender, beforeafter, certainty, normalizedStartDate, normalizedEndDate,
                           normalizedDuration)
    return (dictDate)


def resolverdatebirth(birth, baptism, marriage, death, funeral):
    """
    This function initiates the preparation of birth date information.
    :param birth: birth date information (string)
    :param baptism: baptism date information (string)
    :param marriage: marriage date information (string)
    :param death: death date information (string)
    :param funeral: funeral date information (string)
    :return: corrected birth date information (dictionary)
    """
    birth = resolverdate("birth", birth, birth, baptism, marriage, death, funeral)
    return (birth)


def resolverdatebaptism(birth, baptism, marriage, death, funeral):
    """
    This function initiates the preparation of baptism date information.
    :param birth: birth date information (string)
    :param baptism: baptism date information (string)
    :param marriage: marriage date information (string)
    :param death: death date information (string)
    :param funeral: funeral date information (string)
    :return: corrected baptism date information (dictionary)
    """
    baptism = resolverdate("baptism", baptism, birth, baptism, marriage, death, funeral)
    return (baptism)


def resolverdatemarriage(birth, baptism, marriage, death, funeral):
    """
    This function initiates the preparation of marriage date information.
    :param birth: birth date information (string)
    :param baptism: baptism date information (string)
    :param marriage: marriage date information (string)
    :param death: death date information (string)
    :param funeral: funeral date information (string)
    :return: corrected marriage date information (dictionary)
    """
    marriage = resolverdate("marriage", marriage, birth, baptism, marriage, death, funeral)
    return (marriage)


def resolverdatedeath(birth, baptism, marriage, death, funeral):
    """
    This function initiates the preparation of death date information.
    :param birth: birth date information (string)
    :param baptism: baptism date information (string)
    :param marriage: marriage date information (string)
    :param death: death date information (string)
    :param funeral: funeral date information (string)
    :return: corrected death date information (dictionary)
    """
    death = resolverdate("death", death, birth, baptism, marriage, death, funeral)
    return (death)


def resolverdatefuneral(birth, baptism, marriage, death, funeral):
    """
    This function initiates the preparation of funeral date information.
    :param birth: birth date information (string)
    :param baptism: baptism date information (string)
    :param marriage: marriage date information (string)
    :param death: death date information (string)
    :param funeral: funeral date information (string)
    :return: corrected funeral date information (dictionary)
    """
    funeral = resolverdate("funeral", funeral, birth, baptism, marriage, death, funeral)
    return (funeral)


# load data
encoding = "utf-8-sig"  # "latin-1"
delimiter = "\t"  # ";" #
data = []
with open("karteiLeipzigerFamilien.csv", "r", encoding=encoding) as file:
    for i in csv.DictReader(file, delimiter=delimiter):
        i = json.loads(json.dumps(i))
        # remove special characters
        for number, info in enumerate(i):
            info = info.replace("?", "")
        data.append(i)

# checking whether an ID exists
# if not, the ID is added
for rowNumber, i in enumerate(data):
    try:
        i["id"]
    except KeyError:
        i.update({"id": rowNumber + 1})  # + 1, because 0 must not occur

# special names that do not end in A or E
femaleNamesList = ["Mädchen", "Nelly", "Moly", "Mathildis", "Helen", "Wilha.", "Sophis", "Erato", "Josebeth", "Hel.C.",
                   "Rosinam", "Edeltraut", "Elsisabeth", "Mariam", "Lucy", "Ros.", "Elisaberh", "Sus.", "Eliosabeth",
                   "Emmy", "Elisabeh", "Margareth", "Rebeccam", "Adeheid", "Sab.", "Madchen", "Emilei", "Mari", "Molly",
                   "Theresis", "Amy", "Beatrix", "Felicitas", "Gertrad", "Wilhelmin", "Aelheid", "Magd.", "Melly",
                   "Lilli", "Henriett", "Christin",
                   "Tochter", "Elsiabeth", "Caritas", "Elisabdth", "Molli", "Mathildem", "Theresias", "Apoll.",
                   "[...mädchen...]", "(Elisabeth", "Clothildis",
                   "Rachel", "Hedwig", "Hewig", "Charlott", "Betty", "Nanny", "AX.", "Catharin", "Fides", "Jenny",
                   "Hildegard", "Liddy", "Md.", "Cath.", "Hedwig", "Felizitas", "ME", "Esther", "Sarah", "Charitas",
                   "Gertrud", "Judith", "Lidy", "Doris", "Fanny", "MRos.", "Xa.", "Brunhilde", "Ingrid", "Elisebeth",
                   "Ingeborg", "Elisabeth", "Elisabet", "Adelheid", "Adelh.", "Gertraud", "Walpurg", "Agnes", "Rahel"]
maleNamesList = ["Eustache", "Noa", "Knabe", "Insla", "Bonaventura", "Bernadotte", "Manasse"]

# add empty columns to the data
for rowNumber, i in enumerate(data):
    keysAppend = ["idSpouse1",
                  "idSpouse2",
                  "idSpouse3",
                  "marriageday1",
                  "marriageday2",
                  "marriageday3"]
    for key in keysAppend:
        try:
            data[rowNumber][key]
        except KeyError:
            data[rowNumber].update({key: ""})

# sort by index card name, so that the search of 30 entries before/after works
data = sorted(data, key=lambda k: k["page"])

# data iterate
for rowNumber, i in enumerate(data):
    # set search space
    # relatives are only on one index card
    biggestSizeOfFamily = 30
    if rowNumber - biggestSizeOfFamily < 0:
        localSearchStart = 0
    else:
        localSearchStart = rowNumber - biggestSizeOfFamily
    if rowNumber + biggestSizeOfFamily + 1 > len(data) - 1:
        localSearchEnd = len(data) - 1
    else:
        localSearchEnd = rowNumber + biggestSizeOfFamily + 1

    # status message
    if rowNumber % 1000 == 0:
        print("Status:", round(rowNumber / len(data) * 100, 2), "Prozent")

    # check if father is present
    # if father is not present, then the ID is searched for
    try:
        i["idFather"]
    except KeyError:
        # search if there is a head of family on the same map
        # do not search all, but the 30 before and after
        for j in data[localSearchStart:localSearchEnd]:
            # KLF card number equal and head of family
            if i["page"] == j["page"] and j["Rolle"] == "Familienoberhaupt" and i["Rolle"] == "Kind":
                i.update({"idFather": j["id"]})
                break
            else:
                i.update({"idFather": ""})
    # apply data
    try:
        i["firstnameGiven"]
    except KeyError:
        i.update({"firstnameGiven": i["firstname"].replace("?", "").replace("!", "")})  # remove question mark
    try:
        i["occupation"]
    except KeyError:
        i.update({"occupation": i["Beruf"]})
        # change "K." depending on the role
        # the compgen wiki states that "K." can stand for either "Kaufmann" or "Knabe" (male child)
        if "K." in i["occupation"]:
            # children are "Knabe" (boys), others are "Kaufmann" (merchants)
            if i["Rolle"] == "Kind":
                i["occupation"] = i["occupation"].replace("K.", "Knabe")
            else:
                i["occupation"] = i["occupation"].replace("K.", "Kaufmann")
    try:
        i["surnameGiven"]
    except KeyError:
        i.update({"surnameGiven": i["lastname"]})
    try:
        i["birthday"]
    except KeyError:
        i.update({"birthday": i["Geburtsdatum/Alter"]})
    try:
        i["baptismday"]
    except KeyError:
        i.update({"baptismday": i["Taufdatum"]})
    try:
        i["deathday"]
    except KeyError:
        i.update({"deathday": i["Sterbedatum"]})
    try:
        i["burialday"]
    except KeyError:
        i.update({"burialday": i["Beerd.Datum"]})
    try:
        i["source"]
    except KeyError:
        i.update({"source": i["page"]})
    try:
        i["sex"]
    except KeyError:
        # split name
        # remove spaces at the beginning and end
        nameList = i["firstnameGiven"].strip().split(" ")
        # check only the last first name (the front ones are more often abbreviated)
        # by the role the gender is often fixed
        if i["Rolle"] == "Ehefrau":
            sex = "F"
        elif i["Rolle"] == "Familienoberhaupt":
            sex = "M"
        elif i["Art der Beziehung"] == "Ehefrau":
            sex = "F"
        elif i["Art der Beziehung"] == "Ehemann":
            sex = "M"
        elif i["Art der Beziehung"] == "Vater":
            sex = "M"
        # male special names
        elif nameList[-1] in maleNamesList:
            sex = "M"
        # A or E as last name of given name, then female
        elif nameList[-1][-1:] == "a" or nameList[-1][-1:] == "e":
            sex = "F"
            # if Maria is the last name, check if the first is male
            # there must be more than one first name
            if nameList[-1] == "Maria" and len(nameList) > 1:
                if ((nameList[0][-1] != "a" and nameList[0][-1] != "e") or nameList[0] in maleNamesList) and nameList[
                    0] not in femaleNamesList and nameList[0] != "Joh.":
                    sex = "M"
        # if the last letter is a closed bracket, check the penultimate one (occurs with names in brackets)
        elif nameList[-1][-1:] == ")" and (nameList[-1][-2:-1] == "a" or nameList[-1][-2:-1] == "e"):
            sex = "F"
        # name in exception list, last check, because there also the problem with A/E occurred
        # ignore bracket at the end
        elif nameList != [""] and nameList[-1].replace(")", "") != "" and nameList[-1].replace(")",
                                                                                               "") in femaleNamesList:
            sex = "F"
        else:  # otherwise male
            maleNamesList.append(nameList[-1])
            sex = "M"
        i.update({"sex": sex})

    # wives
    if i["Rolle"] == "Ehefrau":
        # search husband
        for innerindex, j in enumerate(data[localSearchStart:localSearchEnd]):
            # in the case of wives, there are no reference persons, only the card number is the same and the reference to the head of the family of the card
            if i["page"] == j["page"] and j["Rolle"] == "Familienoberhaupt":
                if i["idSpouse1"] == "":
                    i["idSpouse1"] = j["id"]
                    i.update({"marriageday1": i["Heiratsdatum"]})
                elif i["idSpouse2"] == "":
                    i["idSpouse2"] = j["id"]
                    i.update({"marriageday2": i["Heiratsdatum"]})
                elif i["idSpouse3"] == "":
                    i["idSpouse3"] = j["id"]
                    i.update({"marriageday3": i["Heiratsdatum"]})
                # add number for second spouse
                # first marriage date for one spouse does not always have to be the first date for the second spouse
                if data[localSearchStart + innerindex]["idSpouse1"] == "":
                    data[localSearchStart + innerindex]["idSpouse1"] = i["id"]
                    data[localSearchStart + innerindex].update({"marriageday1": i["Heiratsdatum"]})
                elif data[localSearchStart + innerindex]["idSpouse2"] == "":
                    data[localSearchStart + innerindex]["idSpouse2"] = i["id"]
                    data[localSearchStart + innerindex].update({"marriageday2": i["Heiratsdatum"]})
                elif data[localSearchStart + innerindex]["idSpouse3"] == "":
                    data[localSearchStart + innerindex]["idSpouse3"] = i["id"]
                    data[localSearchStart + innerindex].update({"marriageday3": i["Heiratsdatum"]})

    # third parties
    if i["Rolle"] == "Drittperson":
        # father
        if i["Art der Beziehung"] == "Vater":
            # search child of father
            for innerindex, j in enumerate(data[localSearchStart:localSearchEnd]):
                # KLF card number same and IDs must match
                # in addition, it must be excluded that in case of a father with the same name, the father makes himself the father (exclude ID equality)
                if i["page"] == j["page"] and j["id"] == i["Bezugsperson ID"] and j["id"] != i["id"]:
                    # if there is no father with the child, insert ID
                    # if father is already there, add ID to it
                    # in the case of the child, the father is added
                    try:
                        # ID should be added to child (j), not to the father
                        if j["idFather"] == "":
                            j["idFather"] = i["id"]
                            break
                        else:
                            print("Error: Error with a paternity on card", j["page"],
                                  "; ID of the child and the alleged father", j["id"], i["id"])
                            break
                    except KeyError:
                        j["idFather"] = i["id"]
                        break
        # spouse
        if i["Art der Beziehung"] == "Ehemann" or i["Art der Beziehung"] == "Ehefrau":
            # search spouse
            for innerindex, j in enumerate(data[localSearchStart:localSearchEnd]):
                # KLF card number same and ID same
                if i["page"] == j["page"] and j["id"] == i["Bezugsperson ID"]:
                    # if spouseId is empty, then describe this variable, otherwise use the variable spouseid2
                    # ID must be added for two spouses (but for the other partner just the other ID)
                    if i["idSpouse1"] == "":
                        i["idSpouse1"] = j["id"]
                        i.update({"marriageday1": i["Heiratsdatum"]})
                    elif i["idSpouse2"] == "":
                        i["idSpouse2"] = j["id"]
                        i.update({"marriageday2": i["Heiratsdatum"]})
                    elif i["idSpouse3"] == "":
                        i["idSpouse3"] = j["id"]
                        i.update({"marriageday3": i["Heiratsdatum"]})
                    # add the number of the second spouse
                    if data[localSearchStart + innerindex]["idSpouse1"] == "":
                        data[localSearchStart + innerindex]["idSpouse1"] = i["id"]
                        data[localSearchStart + innerindex].update({"marriageday1": i["Heiratsdatum"]})
                    elif data[localSearchStart + innerindex]["idSpouse2"] == "":
                        data[localSearchStart + innerindex]["idSpouse2"] = i["id"]
                        data[localSearchStart + innerindex].update({"marriageday2": i["Heiratsdatum"]})
                    elif data[localSearchStart + innerindex]["idSpouse3"] == "":
                        data[localSearchStart + innerindex]["idSpouse3"] = i["id"]
                        data[localSearchStart + innerindex].update({"marriageday3": i["Heiratsdatum"]})
                    break  # when found

for i in data:
    # for second weddings, delete the information "II. "
    i["marriageday1"] = i["marriageday1"].replace("II. ", "")
    i["marriageday2"] = i["marriageday2"].replace("II. ", "")
    i["marriageday3"] = i["marriageday3"].replace("II. ", "")

    # normalize dates
    birthdict = resolverdatebirth(i["birthday"], i["baptismday"], i["marriageday1"], i["deathday"], i["burialday"])
    baptismdict = resolverdatebaptism(i["birthday"], i["baptismday"], i["marriageday1"], i["deathday"], i["burialday"])
    marriagedict1 = resolverdatemarriage(i["birthday"], i["baptismday"], i["marriageday1"], i["deathday"],
                                         i["burialday"])
    marriagedict2 = resolverdatemarriage(i["birthday"], i["baptismday"], i["marriageday2"], i["deathday"],
                                         i["burialday"])
    marriagedict3 = resolverdatemarriage(i["birthday"], i["baptismday"], i["marriageday3"], i["deathday"],
                                         i["burialday"])
    deathdict = resolverdatedeath(i["birthday"], i["baptismday"], i["marriageday1"], i["deathday"], i["burialday"])
    funeraldict = resolverdatefuneral(i["birthday"], i["baptismday"], i["marriageday1"], i["deathday"], i["burialday"])

    # convert dates if it is a datetime object (only for birth and baptism)
    # convert years (certainty == calculated, typus == relative, day == 1, month == 1) into "BET ... AND ..."-Form
    # sometimes dates are 1.1. and not 01.01.
    try:
        if birthdict["type"] == "point in range" and (
                juliandateToDatetime(birthdict["normalizedStartDate"]).strftime("%d.%m.%Y")[
                :2] == "01" or juliandateToDatetime(birthdict["normalizedStartDate"]).strftime("%d.%m.%Y")[
                               :1] == "1") and (
                juliandateToDatetime(birthdict["normalizedStartDate"]).strftime("%d.%m.%Y")[
                3:5] == "01" or juliandateToDatetime(birthdict["normalizedStartDate"]).strftime("%d.%m.%Y")[
                                2:3] == "1") and juliandateToDatetime(birthdict["normalizedEndDate"]).strftime(
            "%d.%m.%Y")[:2] == "31" and juliandateToDatetime(birthdict["normalizedEndDate"]).strftime("%d.%m.%Y")[
                                        3:5] == "12":
            i.update({"birthday": ("BET " + juliandateToDatetime(birthdict["normalizedStartDate"]).strftime(
                "%d.%m.%Y") + " AND " + juliandateToDatetime(birthdict["normalizedEndDate"]).strftime("%d.%m.%Y"))})
        elif birthdict["type"] == "relative":
            # date of birth only if no valid baptism date is available
            # if it is, the date of birth is calculated because the person is e.g. 60 years old, although the (exact) date of baptism is available
            if baptismdict["type"] == "point in range":
                i.update({"birthday": ""})
            else:
                i.update({"birthday": ("BET " + juliandateToDatetime(birthdict["normalizedEndDate"]).strftime(
                    "%d.%m.%Y") + " AND " + juliandateToDatetime(birthdict["normalizedStartDate"]).strftime(
                    "%d.%m.%Y"))})
        else:
            i.update({"birthday": birthdict["datum"].strftime("%d.%m.%Y")})
    except:  # date is not a datetime object, e.g. because empty ("")
        i.update({"birthday": ""})

    try:
        if baptismdict["type"] == "point in range" and baptismdict["certainty"] == "calculated" and (
                juliandateToDatetime(baptismdict["normalizedStartDate"]).strftime("%d.%m.%Y")[
                :2] == "01" or juliandateToDatetime(baptismdict["normalizedStartDate"]).strftime("%d.%m.%Y")[
                               :1] == "1") and (
                juliandateToDatetime(baptismdict["normalizedStartDate"]).strftime("%d.%m.%Y")[
                3:5] == "01" or juliandateToDatetime(baptismdict["normalizedStartDate"]).strftime("%d.%m.%Y")[
                                2:3] == "1") and juliandateToDatetime(baptismdict["normalizedEndDate"]).strftime(
            "%d.%m.%Y")[:2] == "31" and juliandateToDatetime(baptismdict["normalizedEndDate"]).strftime("%d.%m.%Y")[
                                        3:5] == "12":
            i.update({"baptismday": ("BET " + juliandateToDatetime(baptismdict["normalizedStartDate"]).strftime(
                "%d.%m.%Y") + " AND " + juliandateToDatetime(baptismdict["normalizedEndDate"]).strftime("%d.%m.%Y"))})
        else:
            i.update({"baptismday": baptismdict["datum"].strftime("%d.%m.%Y")})
    except:
        i.update({"baptismday": ""})

    try:
        if marriagedict1["type"] == "point in range" and (
                juliandateToDatetime(marriagedict1["normalizedStartDate"]).strftime("%d.%m.%Y")[
                :2] == "01" or juliandateToDatetime(marriagedict1["normalizedStartDate"]).strftime("%d.%m.%Y")[
                               :1] == "1") and (
                juliandateToDatetime(marriagedict1["normalizedStartDate"]).strftime("%d.%m.%Y")[
                3:5] == "01" or juliandateToDatetime(marriagedict1["normalizedStartDate"]).strftime("%d.%m.%Y")[
                                2:3] == "1") and juliandateToDatetime(marriagedict1["normalizedEndDate"]).strftime(
            "%d.%m.%Y")[:2] == "31" and juliandateToDatetime(marriagedict1["normalizedEndDate"]).strftime(
            "%d.%m.%Y")[3:5] == "12":
            i.update({"marriageday1": ("BET " + juliandateToDatetime(marriagedict1["normalizedStartDate"]).strftime(
                "%d.%m.%Y") + " AND " + juliandateToDatetime(marriagedict1["normalizedEndDate"]).strftime("%d.%m.%Y"))})
        else:
            i.update({"marriageday1": marriagedict1["datum"].strftime("%d.%m.%Y")})
    except:
        i.update({"marriageday1": ""})

    try:
        if marriagedict2["type"] == "point in range" and (
                juliandateToDatetime(marriagedict2["normalizedStartDate"]).strftime("%d.%m.%Y")[
                :2] == "01" or juliandateToDatetime(marriagedict2["normalizedStartDate"]).strftime("%d.%m.%Y")[
                               :1] == "1") and (
                juliandateToDatetime(marriagedict2["normalizedStartDate"]).strftime("%d.%m.%Y")[
                3:5] == "01" or juliandateToDatetime(marriagedict2["normalizedStartDate"]).strftime("%d.%m.%Y")[
                                2:3] == "1") and juliandateToDatetime(marriagedict2["normalizedEndDate"]).strftime(
            "%d.%m.%Y")[:2] == "31" and juliandateToDatetime(marriagedict2["normalizedEndDate"]).strftime(
            "%d.%m.%Y")[3:5] == "12":
            i.update({"marriageday2": ("BET " + juliandateToDatetime(marriagedict2["normalizedStartDate"]).strftime(
                "%d.%m.%Y") + " AND " + juliandateToDatetime(marriagedict2["normalizedEndDate"]).strftime("%d.%m.%Y"))})
        else:
            i.update({"marriageday2": marriagedict2["datum"].strftime("%d.%m.%Y")})
    except:
        i.update({"marriageday2": ""})

    try:
        if marriagedict3["type"] == "point in range" and (
                juliandateToDatetime(marriagedict3["normalizedStartDate"]).strftime("%d.%m.%Y")[
                :2] == "01" or juliandateToDatetime(marriagedict3["normalizedStartDate"]).strftime("%d.%m.%Y")[
                               :1] == "1") and (
                juliandateToDatetime(marriagedict3["normalizedStartDate"]).strftime("%d.%m.%Y")[
                3:5] == "01" or juliandateToDatetime(marriagedict3["normalizedStartDate"]).strftime("%d.%m.%Y")[
                                2:3] == "1") and juliandateToDatetime(marriagedict3["normalizedEndDate"]).strftime(
            "%d.%m.%Y")[:2] == "31" and juliandateToDatetime(marriagedict3["normalizedEndDate"]).strftime(
            "%d.%m.%Y")[3:5] == "12":
            i.update({"marriageday3": ("BET " + juliandateToDatetime(marriagedict3["normalizedStartDate"]).strftime(
                "%d.%m.%Y") + " AND " + juliandateToDatetime(marriagedict3["normalizedEndDate"]).strftime("%d.%m.%Y"))})
        else:
            i.update({"marriageday3": marriagedict3["datum"].strftime("%d.%m.%Y")})
    except:
        i.update({"marriageday3": ""})

    try:
        if deathdict["type"] == "point in range" and juliandateToDatetime(deathdict["normalizedStartDate"]).strftime(
                "%d.%m.%Y")[:2] == "01" and juliandateToDatetime(deathdict["normalizedStartDate"]).strftime("%d.%m.%Y")[
                                            3:5] == "01" and juliandateToDatetime(
            deathdict["normalizedEndDate"]).strftime("%d.%m.%Y")[:2] == "31" and juliandateToDatetime(
            deathdict["normalizedEndDate"]).strftime("%d.%m.%Y")[3:5] == "12":
            i.update({"deathday": ("BET " + juliandateToDatetime(deathdict["normalizedStartDate"]).strftime(
                "%d.%m.%Y") + " AND " + juliandateToDatetime(deathdict["normalizedEndDate"]).strftime("%d.%m.%Y"))})
        else:
            i.update({"deathday": deathdict["datum"].strftime("%d.%m.%Y")})
    except:
        i.update({"deathday": ""})

    try:
        if funeraldict["type"] == "point in range" and (
                juliandateToDatetime(funeraldict["normalizedStartDate"]).strftime("%d.%m.%Y")[
                :2] == "01" or juliandateToDatetime(funeraldict["normalizedStartDate"]).strftime("%d.%m.%Y")[
                               :1] == "1") and (
                juliandateToDatetime(funeraldict["normalizedStartDate"]).strftime("%d.%m.%Y")[
                3:5] == "01" or juliandateToDatetime(funeraldict["normalizedStartDate"]).strftime("%d.%m.%Y")[
                                2:3] == "1") and juliandateToDatetime(funeraldict["normalizedEndDate"]).strftime(
            "%d.%m.%Y")[:2] == "31" and juliandateToDatetime(funeraldict["normalizedEndDate"]).strftime("%d.%m.%Y")[
                                        3:5] == "12":
            i.update({"burialday": ("BET " + juliandateToDatetime(funeraldict["normalizedStartDate"]).strftime(
                "%d.%m.%Y") + " AND " + juliandateToDatetime(funeraldict["normalizedEndDate"]).strftime("%d.%m.%Y"))})
        else:
            i.update({"burialday": funeraldict["datum"].strftime("%d.%m.%Y")})
    except:
        i.update({"burialday": ""})

    # addition of variables that are irrelevant here but occur in the norm form
    addVarList = ["firstnameChange",
                  "surnameChange",
                  "surnameMarriage1",
                  "surnameMarriage2",
                  "surnameMarriage3",
                  "surnameUnknown",
                  "birthplace",
                  "birthplaceGOV",
                  "growthUpPlace",
                  "growthUpPlaceGOV",
                  "baptismplace",
                  "baptismplaceGOV",
                  "marriageplace1",
                  "marriageplace2",
                  "marriageplace3",
                  "marriageplaceGOV1",
                  "marriageplaceGOV2",
                  "marriageplaceGOV3",
                  "ageAtMarriage1",
                  "ageAtMarriage2",
                  "ageAtMarriage3",
                  "divorceday1",
                  "divorceday2",
                  "divorceday3",
                  "deathplace",
                  "deathplaceGOV",
                  "causeOfDeath",
                  "martialStatusAtDeath",
                  "ageAtDeath",
                  "burialplace",
                  "burialplaceGOV"]
    for addVar in addVarList:
        i.update({addVar: ""})

# data iterate
for rowNumber, i in enumerate(data):

    # set search space
    # relatives are only on one index card
    biggestSizeOfFamily = 30
    if rowNumber - biggestSizeOfFamily < 0:
        localSearchStart = 0
    else:
        localSearchStart = rowNumber - biggestSizeOfFamily
    if rowNumber + biggestSizeOfFamily + 1 > len(data) - 1:
        localSearchEnd = len(data) - 1
    else:
        localSearchEnd = rowNumber + biggestSizeOfFamily + 1

    # check if mother is present
    # if mother is not present, then the ID is searched for
    try:
        i["idMother"]
    except KeyError:
        maybeMotherList = []
        # search if there is a wife on the same map
        # do not search all, but the 30 before and after
        for j in data[localSearchStart:localSearchEnd]:
            # KLF card number equal and wife
            if i["page"] == j["page"] and j["Rolle"] == "Ehefrau" and i["Rolle"] == "Kind":
                # selection of date of baptism or birth
                if i["birthday"] != "":
                    birthBaptismInfo = i["birthday"]
                elif i["baptismday"] != "":
                    birthBaptismInfo = i["baptismday"]
                else:
                    birthBaptismInfo = ""

                try:  # date of birth child
                    if "BET" in birthBaptismInfo:
                        bday = birthBaptismInfo[
                               birthBaptismInfo.find("BET ") + 4:birthBaptismInfo.find(" AND ")]  # first date
                    else:
                        bday = birthBaptismInfo
                    datetimeChild = datetime.strptime(bday, "%d.%m.%Y")
                except:
                    try:  # if only the year is specified
                        datetimeChild = datetime.strptime(birthBaptismInfo, "%Y")
                    except:
                        datetimeChild = ""

                try:  # date of death mother
                    if "BET" in j["deathday"]:
                        dday = j["deathday"][j["deathday"].find("AND ") + 4:]  # last date
                    else:
                        dday = j["deathday"]
                    datetimeMother = datetime.strptime(j["deathday"], "%d.%m.%Y")
                except:
                    try:
                        datetimeMother = datetime.strptime(j["deathday"], "%Y")
                    except:
                        try:
                            datetimeMother = datetime.strptime(j["burialday"], "%d.%m.%Y")
                        except:
                            datetimeMother = ""

                try:  # marriage mother
                    datetimeMarriage = datetime.strptime(j["Heiratsdatum"], "%d.%m.%Y")
                except:
                    try:
                        datetimeMarriage = datetime.strptime(j["Heiratsdatum"], "%Y")
                    except:
                        datetimeMarriage = ""

                # create dictionary with the information
                maybeMotherDict = {"id": j["id"],
                                   "datetimeChild": datetimeChild,
                                   "datetimeMother": datetimeMother,
                                   "datetimeMarriage": datetimeMarriage}

                # write potential mothers in a list
                if datetimeChild != "" and datetimeMother != "":  # birth of the child and death of the mother present
                    # birth of the child must not be after the death of the mother
                    if datetimeChild <= datetimeMother:
                        maybeMotherList.append(maybeMotherDict)
                elif datetimeChild != "" and datetimeMarriage != "":  # birth of the child and wedding of the mother present
                    # birth of the child must not be before the wedding (illegitimate children not included in KLF)
                    if datetimeChild >= datetimeMarriage:
                        maybeMotherList.append(maybeMotherDict)
                else:  # if there is no other info, then this can still be the mother
                    maybeMotherList.append(maybeMotherDict)

        # choose right mother
        if len(maybeMotherList) == 1:  # only one comes into question
            i.update({"idMother": maybeMotherList[0]["id"]})
        # in the case of several mothers taking, who died earlier
        elif len(maybeMotherList) > 1:  # several come into question
            idOfRightMother = ""
            # iterate potential mothers
            for mother in maybeMotherList:
                if idOfRightMother == "":  # initial case
                    idOfRightMother = mother["id"]
                    if mother["datetimeMother"] != "":
                        deathOfRightMother = mother["datetimeMother"]  # death date
                    else:
                        deathOfRightMother = datetime.now()
                    if mother["datetimeMarriage"] != "":
                        marriageOfRightMother = mother["datetimeMarriage"]
                    else:  # current date
                        marriageOfRightMother = datetime.now()
                else:
                    # if the date of death (datetimeMother) is smaller, take this mother
                    # if there is no death date, then see which marriage date (datetimeMarriage) is smaller and has the smaller interval
                    # compare only if both infos are available, otherwise the mother without death date cannot overwrite the information
                    if mother["datetimeMother"] != "" and mother["datetimeMother"] < deathOfRightMother:
                        idOfRightMother = mother["id"]  # ID
                        deathOfRightMother = mother["datetimeMother"]  # death date
                        # update marriage date anyway
                        if mother["datetimeMarriage"] != "":
                            marriageOfRightMother = mother["datetimeMarriage"]
                    elif mother["datetimeMother"] == "" and mother["datetimeMarriage"] != "" and mother[
                        "datetimeChild"] != "" and mother["datetimeMarriage"] < mother[
                        "datetimeChild"] and marriageOfRightMother < mother["datetimeMarriage"]:
                        marriageOfRightMother = mother["datetimeMarriage"]
                        idOfRightMother = mother["id"]
            i.update({"idMother": idOfRightMother})
        else:  # no mother
            i.update({"idMother": ""})

# location
# it happens in the data that additional separators are in the data and thereby the ID is shifted one fictitious column further
# this column enters as key "null"
for i in data:
    try:
        i.update({"id": i["null"]})
    except:
        pass

# delete keys that are no longer needed
for rowNumber, i in enumerate(data):
    keysDelete = ["Bezugsperson ID",
                  "page",
                  "lastname",
                  "firstname",
                  "Beruf",
                  "Rolle",
                  "Ort",
                  "GOV-Id",
                  "Bezugsperson",
                  "Art der Beziehung",
                  "Geburtsdatum/Alter",
                  "Taufdatum",
                  "Heiratsdatum",
                  "Sterbedatum",
                  "Beerd.Datum",
                  "Bemerkung",
                  "weiterer Ort",
                  "null"]
    for key in keysDelete:
        i.pop(key, None)

# some IDs are still in a list
# the first list element is relevant
# if the first list element == "" then the second element is relevant
for i in data:
    if type(i["id"]) == list:
        if i["id"][0] == "":
            i["id"] = i["id"][1]
        else:
            i["id"] = i["id"][0]

# save standardized data
fieldnames = data[0].keys()
writer = open("leipzig_normform.csv", "w", newline="", encoding=encoding)
dataWriter = csv.DictWriter(writer, fieldnames=fieldnames, delimiter=delimiter)
dataWriter.writeheader()
dataWriter.writerows(data)
writer.close()

print("Status: Finished")

# print male names
maleNamesList = set(maleNamesList)
for i in maleNamesList:
    print(i)

